@@ -25,10 +25,15 @@ from urllib.parse import urlparse
 from collections import ChainMap
 from .constants import STYLISTS_WORDS,STYLISTS_NEGATIVE_WORDS
 from sqlalchemy import create_engine, text,Table,MetaData,select,update
+from hikerapi import Client as HikerClient
 from api.instagram.models import InstagramUser
 from api.scout.models import Scout,Device
 from django.core.mail import send_mail
+
+
+
 from django.db.models import Q
+from django_tenants.utils import schema_context
 
 class InstagramSpider:
     name = 'instagram'
@@ -49,9 +54,9 @@ class InstagramSpider:
             self.salesrep_instagram_table = Table('sales_rep_salesrep_instagram',self.metadata,autoload_with=self.engine)
 
     
-    def store(self,users,source=1,linked_to='no_one',round=0):
+    def store(self,users,source=1,linked_to='no_one',round=0,item_id=None):
         for user in users:
-            InstagramUser.objects.create(username = user.username,info = user.dict(),source=source,linked_to=linked_to,round=round)
+            InstagramUser.objects.create(username = user.username,info = user.dict(),source=source,linked_to=linked_to,round=round,item_id=item_id)
 
     def is_cursor_available(self):
         is_cursor_available = InstagramUser.objects.filter(Q(username__isnull=True) & Q(cursor__isnull=False))
@@ -214,6 +219,60 @@ class InstagramSpider:
 
         return result
 
+    @schema_context(os.getenv("SCHEMA_NAME"))
+    def scrap_hashtag(self,hashtag):
+        count = Scout.objects.filter(available=True).count()
+
+        if count == 0:
+            return None  # Handle case where no scouts are available
+
+        # Generate a random index
+        random_index = random.randint(0, count - 1)
+
+        # Retrieve a scout at that index using offset
+        random_scout = Scout.objects.filter(available=True)[random_index]
+        client = login_user(random_scout)
+        medias, cursor = client.hashtag_medias_v1_chunk(hashtag, max_amount=3, tab_key='recent')
+        try:
+            for media in medias:
+                try:
+                    media_pk = media.pk
+                except Exception as error:
+                    print(error)
+                    
+                try:
+                    media_info = client.media_info(media_pk)
+                except Exception as error:
+                    print(error)
+                    
+                try:
+                    media_comments = client.media_comments(media_pk)
+                except Exception as error:
+                    print(error)
+                    
+                try:
+                    media_likers = client.media_likers(media_pk)
+                except Exception as error:
+                    print(error)
+                    
+                # Create a DataFrame for the likers and comments    
+                df_likers = pd.DataFrame([{**liker.dict(), "media_link": media_info.id, "media_caption_text": media_info.caption_text} for liker in media_likers])
+                df_comments = pd.DataFrame([{**comment.dict(), "media_link": media_info.id, "media_caption_text": media_info.caption_text} for comment in media_comments])
+                try:
+                    df_comments['username'] = df_comments['user'].apply(lambda x: x['username'] if isinstance(x, dict) else None)
+                except Exception as err:
+                    print("There are no comments attached to media most likely", err)
+                df = pd.concat([df_likers, df_comments],ignore_index=True)
+                # Append the results to the CSV file
+                df.to_csv("prequalified.csv", index=False, mode='a', header=False)
+                for i, row in df.iterrows():
+                    try:
+                        InstagramUser.objects.create(username=row['username'], item_id=media_info.id, is_manually_triggered=True)   
+                    except Exception as error:
+                        print(error)
+        except Exception as error:
+            print(error)
+            
 
     def scrap_inbox(self,scout):
         client = login_user(scout)
@@ -359,10 +418,29 @@ class InstagramSpider:
 
         return result
 
+    
 
+    @schema_context(os.getenv("SCHEMA_NAME"))
     def scrap_media(self, media_links=None):
-        latest_scout = Scout.objects.filter(available=True).first()
-        client = login_user(latest_scout)
+        count = Scout.objects.filter(available=True).count()
+
+        if count == 0:
+            return None  # Handle case where no scouts are available
+
+        # Generate a random index
+        random_index = random.randint(0, count - 1)
+
+        # Retrieve a scout at that index using offset
+        random_scout = Scout.objects.filter(available=True)[random_index]
+        client = None
+        
+        try:
+            client = login_user(random_scout)
+        except Exception as error:
+            logging.warning(error)
+            random_scout.available = False
+            random_scout.save()
+            
 
         # Initialize the CSV file with headers
         header = ['media_link', 'media_caption_text', 'user_id', 'username', 'full_name', 'profile_pic_url', 'is_private', 'is_verified', 'media_count', 'follower_count', 'following_count', 'biography', 'external_url', 'is_business']
@@ -410,14 +488,15 @@ class InstagramSpider:
                 # info_dict = client.user_info_by_username("barbersince").dict()
                 
             try:
-                user_media = client.user_medias(user_id=27971835,amount=1)
+                user_medias = client.user_medias(user_id=27971835,amount=8)
+                user_media = max(user_medias, key=lambda x: x.taken_at)
             except Exception as error:
             #     info_dict.update({"media_id":""})
                 print(error)
             # except Exception as err:
             #     print(err)
             try:
-                media_pk = user_media[0].pk
+                media_pk = user_media.pk
             except Exception as error:
                 print(error)
                 
@@ -454,9 +533,44 @@ class InstagramSpider:
 
         return "successfully scrapped media content"
 
+    @schema_context(os.getenv("SCHEMA_NAME"))
+    def scrap_info_v1(self,delay_before_requests,delay_after_requests,step,accounts,round,index=0):
+        yesterday = timezone.now().date() - timezone.timedelta(days=1)
+        yesterday_start = timezone.make_aware(timezone.datetime.combine(yesterday, timezone.datetime.min.time()))
+        # the instagram users who are manually triggered need to be given first priority
+        instagram_users = None
+        try:
+            response = requests.post(
+                f"{os.getenv('API_URL')}/instagram/getOutreachAccounts/",
+                headers={"Content-Type": "application/json"},
+                data={}
+            )
+            accounts_ = response.json()['accounts']
+            instagram_users = InstagramUser.objects.filter(username__in=[account['igname'] for account in accounts_]).distinct('username')  
+            print(f"found the following number of instagram accounts: {instagram_users.count()}")
+        except Exception as error:
+            logging.warning(error)
+        if instagram_users.exists():
+            pass
+        else:
+            instagram_users = InstagramUser.objects.filter(Q(created_at__gte=yesterday_start) & Q(is_manually_triggered=True)).distinct('username')
+            if instagram_users.exists():
+                pass
+            else:
+                # pick the automatically generated ones
+                instagram_users = InstagramUser.objects.filter(Q(created_at__gte=yesterday_start)).distinct('username')
                 
+        print(len(instagram_users))
+        
+        cl = HikerClient(os.getenv("HIKER_API_KEY"))
+        for i, user in enumerate(instagram_users[index:], start=1):
+            if not user.info:
+                user.info = cl.user_by_username_v1(user.username)
+                user.save()
 
-   
+
+
+    @schema_context(os.getenv("SCHEMA_NAME"))   
     def scrap_info(self,delay_before_requests,delay_after_requests,step,accounts,round,index=0):
         scouts = Scout.objects.filter(available=True)
         scout_index = 0
@@ -479,106 +593,61 @@ class InstagramSpider:
         yesterday = timezone.now().date() - timezone.timedelta(days=1)
         yesterday_start = timezone.make_aware(timezone.datetime.combine(yesterday, timezone.datetime.min.time()))
         # the instagram users who are manually triggered need to be given first priority
-        instagram_users = InstagramUser.objects.filter(Q(created_at__gte=yesterday_start) & Q(is_manually_triggered=True)).distinct('username')
+        instagram_users = None
+        try:
+            response = requests.post(
+                f"{os.getenv('API_URL')}/instagram/getOutreachAccounts/",
+                headers={"Content-Type": "application/json"},
+                data={}
+            )
+            accounts_ = response.json()['accounts']
+            instagram_users = InstagramUser.objects.filter(username__in=[account['igname'] for account in accounts_])
+            print(f"found the following number of instagram accounts: {instagram_users.count()}")
+        except Exception as error:
+            logging.warning(error)
         if instagram_users.exists():
-            # manually qualify and assign those accounts
-            headers = {
-                'Content-Type': 'application/json'
-            }
-            for user in instagram_users:
-                # check if account or thread exists otherwise pull the inbox afresh
-                check_accounts_endpoint = "https://api.booksy.us.boostedchat.com/v1/instagram/checkAccountExists/"
-                check_threads_endpoint = "https://api.booksy.us.boostedchat.com/v1/instagram/checkThreadExists/"
-                check_data = {
-                    "username": user.username
-                }
-                check_account_response = requests.post(check_accounts_endpoint,data=check_data)
-                check_thread_response = requests.post(check_threads_endpoint,data=check_data)
-                if check_account_response.json()['exists'] and check_thread_response.json()['exists']:
-                    # manually trigger, qualify, and assign
-                    # perform inbound triggering
-                    # inbound_trigger_data = {
-                    #     "username": user.username
-                    # }
-                    # response = requests.post("https://api.booksy.us.boostedchat.com/v1/instagram/account/manually-trigger/",data=inbound_trigger_data)
-                    # if response.status_code in [200,201]:
-                    #     print(f"Account-----{user.username} successfully triggered")
-
-                    # # perform inbound qualifying
-                    # inbound_qualify_data = {
-                    #     "username": user.username,
-                    #     "qualify_flag": True,
-                    #     "relevant_information": user.relevant_information,
-                    #     "scraped":True
-                    # }
-                    # response = requests.post("https://api.booksy.us.boostedchat.com/v1/instagram/account/qualify-account/",data=inbound_qualify_data)
-                    # if response.status_code in [200,201]:
-                    #     print(f"Account-----{user.username} successfully qualified")
-
-                    # perform outbound qualifying
-                    print("user in database")
-                    user.qualified = True
-                    user.save()
-
-
-                    # perform salesrep assignment
-                    # endpoint = "https://api.booksy.us.boostedchat.com/v1/sales/assign-salesrep/"
-                    # payload = {"username": ""}
-                    # try:
-                    #     response = requests.post(endpoint, data=json.dumps(payload), headers=headers)
-                    #     response.raise_for_status()  # Raise an exception for HTTP errors
-                    #     print(f"Account-----{user.username} successfully assigned")
-                    # except requests.exceptions.RequestException as e:
-                    #     print( {"error": str(e)})
-                else:
-                    pass
-                    # fetch the direct inbox items
-
-                    # username = 'blendscrafters'
-                    # endpoint = "https://mqtt.booksy.us.boostedchat.com"
-                    # Send a POST request to the fetchDirectInbox endpoint
-                    
-                    # response = requests.post(f'{endpoint}/fetchDirectInbox', json={'username_from': username})
-                    
-                    # # Check the status code of the response
-                    # if response.status_code == 200:
-                    #     # Print the response JSON
-                    #     print("all is well")
-                    #     print(json.dumps(response.json(), indent=2))
-                    #     inbox_data = response.json()
-                    #     inbox_dataset = self.extract_direct_inbox_data(inbox_data)
-                    #     print(inbox_dataset)
-                        
-                    # else:
-                    #     print(f'Request failed with status code {response.status_code}')
+            pass
         else:
-            # pick the automatically generated ones
-            instagram_users = InstagramUser.objects.filter(Q(created_at__gte=yesterday_start))
-
+            instagram_users = InstagramUser.objects.filter(Q(created_at__gte=yesterday_start) & Q(is_manually_triggered=True)).distinct('username')
+            if instagram_users.exists():
+                pass
+            else:
+                # pick the automatically generated ones
+                instagram_users = InstagramUser.objects.filter(Q(created_at__gte=yesterday_start)).distinct('username')
+                
         print(len(instagram_users))
         
         for i, user in enumerate(instagram_users[index:], start=1):
-            print(i)
+            # only append info for the qualified ones
+
+            if user.info and "biography" in user.info:
+                continue
             if user.username:
                 time.sleep(random.randint(delay_before_requests,delay_before_requests+step))
                 try:
 
-                    info_dict = client.user_info_by_username(user.username).dict()
+                    info_dict_ = client.user_info_by_username_v1(user.username).model_dump_json()
+                    info_dict = json.loads(info_dict_)
+                    if user.item_id:
+                        info_dict.update({"media_id":user.item_id})
                     try:
-                        user_medias = client.user_medias(info_dict.get("pk"),amount=1)
                         # comment = self.generate_comment(user_medias[0],user.username)
                         # info_dict.update({"media_comment":comment})
-                        media_res = []
-                        for user_media in user_medias:
-                            if user_media:
-                                media_info_ = client.media_info(user_media.id)
-                                media_res.append({
-                                    "media_id":media_info_.id,
-                                    "media_url":media_info_.thumbnail_url,
-                                    "media_caption":media_info_.caption_text,
-                                    # "media_taken_at":media_info_.taken_at
-                                })
-                        info_dict.update({"medias":media_res})
+                        if not info_dict.get("is_private"):
+                            user_medias = client.user_medias_v1(info_dict.get("pk"),amount=1)
+                            media_res = []
+                            for user_media in user_medias:
+                                if user_media:
+                                    media_info_ = client.media_info_v1(user_media.id)
+                                    media_res.append({
+                                        "media_id":media_info_.id,
+                                        "media_url":media_info_.thumbnail_url.unicode_string(),
+                                        "media_caption":media_info_.caption_text,
+                                        # "media_taken_at":media_info_.taken_at
+                                    })
+                            info_dict.update({"medias":media_res})
+                        else:
+                            logging.warning("Private accounts don't allow us to view their media")
                     except Exception as error:
                         info_dict.update({"media_id":""})
                         print(error)
@@ -589,42 +658,6 @@ class InstagramSpider:
                         user.save()
                     except Exception as err:
                         print(f"failed to save user------>{err}")
-                    # try:
-                    #     account_dict = {
-                    #         "igname": user.username,
-                    #         "is_manually_triggered":True
-                    #     }
-                    #     # Save account data
-                    #     response = requests.post(
-                    #         "https://api.booksy.us.boostedchat.com/v1/instagram/account/",
-                    #         headers=headers,
-                    #         data=json.dumps(account_dict)
-                    #     )
-                    #     account = response.json()
-                    #     # Save outsourced data
-                    #     outsourced_dict = {
-                    #         "results": {
-                    #             **user.info
-                    #         },
-                    #         "source": "instagram"
-                    #     }
-                    #     response = requests.post(
-                    #         f"https://api.booksy.us.boostedchat.com/v1/instagram/account/{account['id']}/add-outsourced/",
-                    #         headers=headers,
-                    #         data=json.dumps(outsourced_dict)
-                    #     )
-                    #     inbound_qualify_data = {
-                    #         "username": user.username,
-                    #         "qualify_flag": True,
-                    #         "relevant_information": user.relevant_information,
-                    #         "scraped":True
-                    #     }
-                    #     response = requests.post("https://api.booksy.us.boostedchat.com/v1/instagram/account/qualify-account/",data=inbound_qualify_data)
-
-                    #     if response.status_code in [200,201]:
-                    #         print(f"Account-----{user.username} successfully qualified")
-                    # except Exception as error:
-                    #     print(error)
                         
                 except Exception as error:
                     user.outsourced_id_pointer=True
